/*
** Module   : UnimplementedFeature.java
** Abstract : helper class to cleanly log runtime usage of code that is unimplemented or partial
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description----------------------------------
** 001 GES 20141029 Created initial version.
*/

package com.goldencode.p2j.util;

import java.util.logging.*;

/**
 * Helper class to cleanly log runtime usage of code that is unimplemented or partial.
 */
public class UnimplementedFeature
{
   /** Logger */
   private static final Logger logger = LogHelper.getLogger(UnimplementedFeature.class);
   
   /**
    * Log the given message along with the given stack trace and a prefix that indicates that
    * a feature was used at runtime for which the runtime support is completely missing. The
    * logging is unconditionally done (there is no logging level checked) with the level set
    * to <code>SEVERE</code>.
    * <p>
    * Since a call stack will be created and logged, the location of the missing feature will
    * already be readily apparent.
    *
    * @param    msg
    *           A description of the missing feature.
    */
   public static void missing(String msg)
   {
      log("MISSING", msg, new RuntimeException());
   }
   
   /**
    * Log the given message along with the given stack trace and a prefix that indicates that
    * a feature was used at runtime for which the runtime support is only partially implemented.
    * The logging is unconditionally done (there is no logging level checked) with the level
    * set to <code>SEVERE</code>.
    * <p>
    * Since a call stack will be created and logged, the location of the partial feature will
    * already be readily apparent.
    *
    * @param    msg
    *           A description of the missing feature.
    */
   public static void partial(String msg)
   {
      log("PARTIAL", msg, new RuntimeException());
   }
   
   /**
    * Log the given message along with the given stack trace and a prefix that indicates that
    * a feature was used at runtime for which the runtime support has a TODO remaining. The
    * logging is unconditionally done (there is no logging level checked) with the level set
    * to <code>SEVERE</code>.
    * <p>
    * Since a call stack will be created and logged, the location of the TODO will already be
    * readily apparent.
    *
    * @param    msg
    *           A description of the missing feature.
    */
   public static void todo(String msg)
   {
      log("TODO", msg, new RuntimeException());
   }
   
   /**
    * Logging helper method that logs a <code>SEVERE</code> message.
    *
    * @param    code
    *           Describes the type of unimplemented feature.
    * @param    msg
    *           The caller-defined message.
    * @param    exc
    *           Exception that contains the call stack to include in the log entry.
    */
   private static void log(String code, String msg, RuntimeException exc)
   {
      String txt = String.format("%s: %s", code, msg);
      
      // the code is passed here but not honored, so we add it explicitly above
      logger.logp(Level.SEVERE, "UnimplementedFeature", code, LogHelper.generate(txt), exc);
   }
}
